@@ -7,7 +7,7 @@
 ******************************************************************************/
 
 /*
- * $Id: fixio.c,v 1.6 2001/12/30 05:12:04 jason Exp $
+ * $Id: fixio.c,v 1.7 2003/08/26 05:34:04 jason Exp $
  */
 
 #include <stdio.h>
@@ -193,7 +193,7 @@ void fwrite_type(slong **data,int ftype,int nchan,int nitem,shn_file *this_shn)
       nwrite = nitem;
     }
     else
-      debug("Buffer overrun in fwrite_type() [case 1]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
+      shn_debug("Buffer overrun in fwrite_type() [case 1]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
     break;
   case TYPE_S16HL:
   case TYPE_U16HL:
@@ -205,7 +205,7 @@ void fwrite_type(slong **data,int ftype,int nchan,int nitem,shn_file *this_shn)
         nwrite = nitem;
       }
       else
-        debug("Buffer overrun in fwrite_type() [case 2]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
+        shn_debug("Buffer overrun in fwrite_type() [case 2]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
     }
     else
     {
@@ -216,7 +216,7 @@ void fwrite_type(slong **data,int ftype,int nchan,int nitem,shn_file *this_shn)
         nwrite = nitem;
       }
       else
-        debug("Buffer overrun in fwrite_type() [case 3]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
+        shn_debug("Buffer overrun in fwrite_type() [case 3]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
     }
     break;
   case TYPE_S16LH:
@@ -230,7 +230,7 @@ void fwrite_type(slong **data,int ftype,int nchan,int nitem,shn_file *this_shn)
         nwrite = nitem;
       }
       else
-        debug("Buffer overrun in fwrite_type() [case 4]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
+        shn_debug("Buffer overrun in fwrite_type() [case 4]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
     }
     else
     {
@@ -240,13 +240,13 @@ void fwrite_type(slong **data,int ftype,int nchan,int nitem,shn_file *this_shn)
         nwrite = nitem;
       }
       else
-        debug("Buffer overrun in fwrite_type() [case 5]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
+        shn_debug("Buffer overrun in fwrite_type() [case 5]: %d bytes to read, but only %d bytes are available",datasize*nchan*nitem,bufAvailable);
     }
     break;
   }
 
   if(nwrite != nitem)
-    fatal_error(this_shn,"Failed to write decompressed stream -\npossible corrupt or truncated file");
+    shn_error_fatal(this_shn,"Failed to write decompressed stream -\npossible corrupt or truncated file");
 }
 
 /*************/
