@@ -1,5 +1,5 @@
 /*  gtk.c - functions to build and display GTK windows
- *  Copyright (C) 2000-2003  Jason Jordan <shnutils@freeshell.org>
+ *  Copyright (C) 2000-2004  Jason Jordan <shnutils@freeshell.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: gtk.c,v 1.20 2003/08/26 05:34:04 jason Exp $
+ * $Id: gtk.c,v 1.25 2004/05/02 12:09:42 jason Exp $
  */
 
 #include <stdlib.h>
@@ -87,7 +87,7 @@ void shn_display_about(void)
 	about_box = xmms_show_message(
 		(gchar *) "About " PACKAGE,
 		(gchar *) PACKAGE " version " VERSION "\n"
-			  "Copyright 2000-2003 Jason Jordan <shnutils@freeshell.org>\n"
+			  "Copyright 2000-2004 Jason Jordan <shnutils@freeshell.org>\n"
 			  "Portions Copyright (C) 1992-1995 Tony Robinson\n"
 			  "\n"
 			  "shorten utilities pages:\n"
@@ -443,9 +443,13 @@ void load_shntextfile(char *filename,int num,GtkWidget *shntxt_notebook)
 	int nchars;
 	GtkWidget *shntxt_frame,
 		  *shntxt_vbox,
+		  *shntxt_vbox_inner,
 		  *shntxt_text,
 		  *shntxt_table,
-		  *shntxt_vscrollbar;
+		  *shntxt_vscrollbar,
+		  *shntxt_filename_hbox,
+		  *shntxt_filename_entry,
+		  *shntxt_filename_label;
 
 	shn_debug("Loading text file '%s'",filename);
 
@@ -462,10 +466,28 @@ void load_shntextfile(char *filename,int num,GtkWidget *shntxt_notebook)
 	gtk_container_set_border_width(GTK_CONTAINER(shntxt_frame), 10);
 	gtk_box_pack_start(GTK_BOX(shntxt_vbox), shntxt_frame, TRUE, TRUE, 0);
 
+	shntxt_vbox_inner = gtk_vbox_new(FALSE, 10);
+	gtk_container_set_border_width(GTK_CONTAINER(shntxt_vbox_inner), 5);
+	gtk_container_add(GTK_CONTAINER(shntxt_frame), shntxt_vbox_inner);
+
+/* begin filename */
+	shntxt_filename_hbox = gtk_hbox_new(FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(shntxt_vbox_inner), shntxt_filename_hbox, FALSE, TRUE, 0);
+
+	shntxt_filename_label = gtk_label_new((gchar *)"Filename:");
+	gtk_box_pack_start(GTK_BOX(shntxt_filename_hbox), shntxt_filename_label, FALSE, TRUE, 0);
+	shntxt_filename_entry = gtk_entry_new();
+	gtk_editable_set_editable(GTK_EDITABLE(shntxt_filename_entry), FALSE);
+	gtk_box_pack_start(GTK_BOX(shntxt_filename_hbox), shntxt_filename_entry, TRUE, TRUE, 0);
+
+	gtk_entry_set_text(GTK_ENTRY(shntxt_filename_entry), filename);
+	gtk_editable_set_position(GTK_EDITABLE(shntxt_filename_entry), -1);
+/* end filename */
+
 	shntxt_text = gtk_text_new(NULL,NULL);
 
 	shntxt_table = gtk_table_new(2,2,FALSE);
-	gtk_container_add(GTK_CONTAINER(shntxt_frame), shntxt_table);
+	gtk_container_add(GTK_CONTAINER(shntxt_vbox_inner), shntxt_table);
 
 	shntxt_vscrollbar = gtk_vscrollbar_new(GTK_TEXT(shntxt_text)->vadj);
 
@@ -480,8 +502,12 @@ void load_shntextfile(char *filename,int num,GtkWidget *shntxt_notebook)
 	gtk_widget_show(shntxt_table);
 	gtk_widget_show(shntxt_frame);
 	gtk_widget_show(shntxt_vbox);
+	gtk_widget_show(shntxt_vbox_inner);
+	gtk_widget_show(shntxt_filename_hbox);
+	gtk_widget_show(shntxt_filename_entry);
+	gtk_widget_show(shntxt_filename_label);
 
-	if ((f = fopen(filename,"r")))
+	if ((f = fopen(filename,"rb")))
 	{
 		nchars = fread(buffer,1,1024,f);
 		while (nchars > 0)
@@ -582,18 +608,29 @@ void shn_display_info(shn_file *tmp_file)
 	char details[BUF_SIZE];
 	char props_label[BUF_SIZE];
 	char details_label[BUF_SIZE];
+	char misalignment[8];
+	char seektable_revision[8];
+	char id3v2_info[32];
 	GtkWidget *info_win,
 		  *info_notebook,
 		  *props_hbox,
 		  *props_vbox,
+		  *props_vbox_inner,
 		  *props_frame,
 		  *props_label_left,
 		  *props_label_right,
+		  *props_filename_hbox,
+		  *props_filename_label,
+		  *props_filename_entry,
 		  *details_hbox,
 		  *details_vbox,
+		  *details_vbox_inner,
 		  *details_frame,
 		  *details_label_left,
 		  *details_label_right,
+		  *details_filename_hbox,
+		  *details_filename_label,
+		  *details_filename_entry,
 		  *main_vbox,
 		  *info_bbox,
 		  *info_ok;
@@ -608,30 +645,43 @@ void shn_display_info(shn_file *tmp_file)
 		strrchr(tmp_file->wave_header.filename,'/') + 1 :
 		tmp_file->wave_header.filename);
 
+	shn_snprintf(misalignment,8,"%d",tmp_file->wave_header.data_size % CD_BLOCK_SIZE);
+
+	if (NO_SEEK_TABLE != tmp_file->seek_header.version)
+		shn_snprintf(seektable_revision,8,"%d",tmp_file->seek_header.version);
+
+	shn_snprintf(id3v2_info,32,"yes (%ld bytes)",tmp_file->wave_header.id3v2_tag_size);
+
 	shn_snprintf(props,BUF_SIZE,
-        "\n"
+		"%s\n"
 		"%s\n"
 		"%s\n"
 		"%0.4f\n"
 		"\n"
 		"%s\n"
 		"%s\n"
+		"%s bytes\n"
 		"%s\n"
 		"\n"
 		"%s\n"
-		"%s"
-		"\n",
+		"%s\n"
+		"\n"
+		"%s",
 		tmp_file->wave_header.m_ss,
-		(tmp_file->vars.seek_table_entries == NO_SEEK_TABLE) ? "no":"yes",
+		(tmp_file->vars.seek_table_entries == NO_SEEK_TABLE)?"no":"yes",
+		(tmp_file->seek_header.version == NO_SEEK_TABLE)?"n/a":seektable_revision,
 		(double)tmp_file->wave_header.actual_size/(double)tmp_file->wave_header.total_size,
 		(PROB_NOT_CD(tmp_file->wave_header))?"no":"yes",
 		(PROB_NOT_CD(tmp_file->wave_header))?"n/a":((PROB_BAD_BOUND(tmp_file->wave_header))?"no":"yes"),
+		(PROB_NOT_CD(tmp_file->wave_header))?"n/a":misalignment,
 		(PROB_NOT_CD(tmp_file->wave_header))?"n/a":((PROB_TOO_SHORT(tmp_file->wave_header))?"no":"yes"),
 		(PROB_HDR_NOT_CANONICAL(tmp_file->wave_header))?"yes":"no",
-		(PROB_EXTRA_CHUNKS(tmp_file->wave_header))?"yes":"no"
+		(PROB_EXTRA_CHUNKS(tmp_file->wave_header))?"yes":"no",
+		(tmp_file->wave_header.file_has_id3v2_tag)?id3v2_info:"no"
 		);
 
 	shn_snprintf(details,BUF_SIZE,
+		"\n"
 		"0x%04x (%s)\n"
 		"%hu\n"
 		"%hu\n"
@@ -679,23 +729,42 @@ void shn_display_info(shn_file *tmp_file)
 	gtk_container_set_border_width(GTK_CONTAINER(props_frame), 10);
 	gtk_box_pack_start(GTK_BOX(props_vbox), props_frame, TRUE, TRUE, 0);
 
+	props_vbox_inner = gtk_vbox_new(FALSE, 10);
+	gtk_container_set_border_width(GTK_CONTAINER(props_vbox_inner), 5);
+	gtk_container_add(GTK_CONTAINER(props_frame), props_vbox_inner);
+
+/* begin filename */
+	props_filename_hbox = gtk_hbox_new(FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(props_vbox_inner), props_filename_hbox, FALSE, TRUE, 0);
+
+	props_filename_label = gtk_label_new((gchar *)"Filename:");
+	gtk_box_pack_start(GTK_BOX(props_filename_hbox), props_filename_label, FALSE, TRUE, 0);
+	props_filename_entry = gtk_entry_new();
+	gtk_editable_set_editable(GTK_EDITABLE(props_filename_entry), FALSE);
+	gtk_box_pack_start(GTK_BOX(props_filename_hbox), props_filename_entry, TRUE, TRUE, 0);
+
+	gtk_entry_set_text(GTK_ENTRY(props_filename_entry), tmp_file->wave_header.filename);
+	gtk_editable_set_position(GTK_EDITABLE(props_filename_entry), -1);
+/* end filename */
+
 	props_hbox = gtk_hbox_new(FALSE, 10);
-	gtk_container_set_border_width(GTK_CONTAINER(props_hbox), 5);
-	gtk_container_add(GTK_CONTAINER(props_frame), props_hbox);
+	gtk_box_pack_start(GTK_BOX(props_vbox_inner), props_hbox, TRUE, TRUE, 0);
 
 	props_label_left = gtk_label_new((gchar *)
-		"\n"
 		"Length:\n"
 		"Seekable:\n"
+		"Seek table revision:\n"
 		"Compression ratio:\n"
 		"CD-quality properties:\n"
 		"  CD-quality:\n"
 		"  Cut on sector boundary:\n"
+		"  Sector misalignment:\n"
 		"  Long enough to be burned:\n"
 		"WAVE properties:\n"
 		"  Non-canonical header:\n"
-		"  Extra RIFF chunks:"
-		"\n"
+		"  Extra RIFF chunks:\n"
+		"Possible problems:\n"
+		"  File contains ID3v2 tag:"
 		);
 
 	props_label_right = gtk_label_new((gchar *)props);
@@ -720,11 +789,29 @@ void shn_display_info(shn_file *tmp_file)
 	gtk_container_set_border_width(GTK_CONTAINER(details_frame), 10);
 	gtk_box_pack_start(GTK_BOX(details_vbox), details_frame, TRUE, TRUE, 0);
 
+	details_vbox_inner = gtk_vbox_new(FALSE, 10);
+	gtk_container_set_border_width(GTK_CONTAINER(details_vbox_inner), 5);
+	gtk_container_add(GTK_CONTAINER(details_frame), details_vbox_inner);
+
+/* begin filename */
+	details_filename_hbox = gtk_hbox_new(FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(details_vbox_inner), details_filename_hbox, FALSE, TRUE, 0);
+
+	details_filename_label = gtk_label_new((gchar *)"Filename:");
+	gtk_box_pack_start(GTK_BOX(details_filename_hbox), details_filename_label, FALSE, TRUE, 0);
+	details_filename_entry = gtk_entry_new();
+	gtk_editable_set_editable(GTK_EDITABLE(details_filename_entry), FALSE);
+	gtk_box_pack_start(GTK_BOX(details_filename_hbox), details_filename_entry, TRUE, TRUE, 0);
+
+	gtk_entry_set_text(GTK_ENTRY(details_filename_entry), tmp_file->wave_header.filename);
+	gtk_editable_set_position(GTK_EDITABLE(details_filename_entry), -1);
+/* end filename */
+
 	details_hbox = gtk_hbox_new(FALSE, 10);
-	gtk_container_set_border_width(GTK_CONTAINER(details_hbox), 5);
-	gtk_container_add(GTK_CONTAINER(details_frame), details_hbox);
+	gtk_box_pack_start(GTK_BOX(details_vbox_inner), details_hbox, TRUE, TRUE, 0);
 
 	details_label_left = gtk_label_new((gchar *)
+		"\n"
 		"WAVE format:\n"
 		"Channels:\n"
 		"Bits per sample:\n"
@@ -770,10 +857,18 @@ void shn_display_info(shn_file *tmp_file)
 
 	gtk_widget_show(props_frame);
 	gtk_widget_show(props_hbox);
+	gtk_widget_show(props_vbox_inner);
 	gtk_widget_show(props_vbox);
+	gtk_widget_show(props_filename_hbox);
+	gtk_widget_show(props_filename_entry);
+	gtk_widget_show(props_filename_label);
 	gtk_widget_show(details_frame);
 	gtk_widget_show(details_hbox);
+	gtk_widget_show(details_vbox_inner);
 	gtk_widget_show(details_vbox);
+	gtk_widget_show(details_filename_hbox);
+	gtk_widget_show(details_filename_entry);
+	gtk_widget_show(details_filename_label);
 
 /* begin any text files pages */
 
