@@ -1,5 +1,5 @@
 /*  shn.c - main functions for xmms-shn
- *  Copyright (C) 2000-2003  Jason Jordan <shnutils@freeshell.org>
+ *  Copyright (C) 2000-2004  Jason Jordan <shnutils@freeshell.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: shn.c,v 1.23 2003/08/26 05:34:04 jason Exp $
+ * $Id: shn.c,v 1.35 2004/05/04 02:26:36 jason Exp $
  */
 
 #include <stdio.h>
@@ -182,7 +182,7 @@ int get_wave_header(shn_file *this_shn)
 {
   slong  **buffer = NULL, **offset = NULL;
   slong  lpcqoffset = 0;
-  int   version = FORMAT_VERSION;
+  int   version = FORMAT_VERSION, bitshift = 0;
   int   ftype = TYPE_EOF;
   char  *magic = MAGIC;
   int   blocksize = DEFAULT_BLOCK_SIZE, nchan = DEFAULT_NCHAN;
@@ -191,7 +191,7 @@ int get_wave_header(shn_file *this_shn)
   int   cmd;
   int   internal_ftype;
   int   cklen;
-  int   retval = 1;
+  int   retval = 0;
 
   if (!init_decode_state(this_shn))
     return 0;
@@ -204,12 +204,15 @@ int get_wave_header(shn_file *this_shn)
 #ifdef STRICT_FORMAT_COMPATABILITY
     if(FORMAT_VERSION < 2)
     {
-      for(i = 0; i < strlen(magic); i++)
-	if(getc_exit(this_shn->vars.fd) != magic[i])
+      for(i = 0; i < strlen(magic); i++) {
+        if(getc_exit(this_shn->vars.fd) != magic[i])
           return 0;
+        this_shn->vars.bytes_read++;
+      }
 
       /* get version number */
       version = getc_exit(this_shn->vars.fd);
+      this_shn->vars.bytes_read++;
     }
     else
 #endif /* STRICT_FORMAT_COMPATABILITY */
@@ -219,24 +222,25 @@ int get_wave_header(shn_file *this_shn)
       version = MAX_VERSION + 1;
       while(version > MAX_VERSION)
       {
-  	int byte = getc(this_shn->vars.fd);
-	if(byte == EOF)
+        int byte = getc(this_shn->vars.fd);
+        this_shn->vars.bytes_read++;
+        if(byte == EOF)
           return 0;
-	if(magic[nscan] != '\0' && byte == magic[nscan])
+        if(magic[nscan] != '\0' && byte == magic[nscan])
           nscan++;
-	else
-        if(magic[nscan] == '\0' && byte <= MAX_VERSION)
-          version = byte;
-	else
+        else
+          if(magic[nscan] == '\0' && byte <= MAX_VERSION)
+            version = byte;
+        else
         {
-	  if(byte == magic[0])
-  	    nscan = 1;
-	  else
+          if(byte == magic[0])
+            nscan = 1;
+          else
           {
-	    nscan = 0;
-	  }
-	  version = MAX_VERSION + 1;
-	}
+            nscan = 0;
+          }
+         version = MAX_VERSION + 1;
+        }
       }
     }
 
@@ -260,12 +264,17 @@ int get_wave_header(shn_file *this_shn)
 
     /* has the user requested a change in file type? */
     if(internal_ftype != ftype) {
-      if(ftype == TYPE_EOF)
-	ftype = internal_ftype;    /*  no problems here */
-      else             /* check that the requested conversion is valid */
-	if(internal_ftype == TYPE_AU1 || internal_ftype == TYPE_AU2 ||
-	   internal_ftype == TYPE_AU3 || ftype == TYPE_AU1 ||ftype == TYPE_AU2 || ftype == TYPE_AU3)
+      if(ftype == TYPE_EOF) {
+        ftype = internal_ftype;    /*  no problems here */
+      }
+      else {           /* check that the requested conversion is valid */
+        if(internal_ftype == TYPE_AU1 || internal_ftype == TYPE_AU2 ||
+           internal_ftype == TYPE_AU3 || ftype == TYPE_AU1 ||ftype == TYPE_AU2 || ftype == TYPE_AU3)
+        {
+          retval = 0;
           goto got_enough_data;
+        }
+      }
     }
 
     nchan = UINT_GET(CHANSIZE, this_shn);
@@ -332,21 +341,183 @@ int get_wave_header(shn_file *this_shn)
     chan = 0;
     while(1)
     {
-        cmd = uvar_get(FNSIZE,this_shn);
+      this_shn->vars.reading_function_code = 1;
+      cmd = uvar_get(FNSIZE,this_shn);
+      this_shn->vars.reading_function_code = 0;
+
         switch(cmd)
         {
+          case FN_ZERO:
+          case FN_DIFF0:
+          case FN_DIFF1:
+          case FN_DIFF2:
+          case FN_DIFF3:
+          case FN_QLPC:
+          {
+            slong coffset, *cbuffer = buffer[chan];
+            int resn = 0, nlpc, j;
+
+            if(cmd != FN_ZERO)
+            {
+              resn = uvar_get(ENERGYSIZE,this_shn);
+              if (this_shn->vars.fatal_error) {
+                retval = 0;
+                goto got_enough_data;
+              }
+              /* this is a hack as version 0 differed in definition of var_get */
+              if(version == 0)
+                resn--;
+            }
+
+            /* find mean offset : N.B. this code duplicated */
+            if(nmean == 0)
+              coffset = offset[chan][0];
+            else
+            {
+              slong sum = (version < 2) ? 0 : nmean / 2;
+              for(i = 0; i < nmean; i++)
+                sum += offset[chan][i];
+              if(version < 2)
+                coffset = sum / nmean;
+              else
+                coffset = ROUNDEDSHIFTDOWN(sum / nmean, bitshift);
+            }
+
+            switch(cmd)
+            {
+              case FN_ZERO:
+                for(i = 0; i < blocksize; i++)
+                  cbuffer[i] = 0;
+                break;
+              case FN_DIFF0:
+                for(i = 0; i < blocksize; i++) {
+                  cbuffer[i] = var_get(resn,this_shn) + coffset;
+                  if (this_shn->vars.fatal_error) {
+                    retval = 0;
+                    goto got_enough_data;
+                  }
+                }
+                break;
+              case FN_DIFF1:
+                for(i = 0; i < blocksize; i++) {
+                  cbuffer[i] = var_get(resn,this_shn) + cbuffer[i - 1];
+                  if (this_shn->vars.fatal_error) {
+                    retval = 0;
+                    goto got_enough_data;
+                  }
+                }
+                break;
+              case FN_DIFF2:
+                for(i = 0; i < blocksize; i++) {
+                  cbuffer[i] = var_get(resn,this_shn) + (2 * cbuffer[i - 1] -	cbuffer[i - 2]);
+                  if (this_shn->vars.fatal_error) {
+                    retval = 0;
+                    goto got_enough_data;
+                  }
+                }
+                break;
+              case FN_DIFF3:
+                for(i = 0; i < blocksize; i++) {
+                  cbuffer[i] = var_get(resn,this_shn) + 3 * (cbuffer[i - 1] -  cbuffer[i - 2]) + cbuffer[i - 3];
+                  if (this_shn->vars.fatal_error) {
+                    retval = 0;
+                    goto got_enough_data;
+                  }
+                }
+                break;
+              case FN_QLPC:
+                nlpc = uvar_get(LPCQSIZE,this_shn);
+                if (this_shn->vars.fatal_error) {
+                  retval = 0;
+                  goto got_enough_data;
+                }
+
+                for(i = 0; i < nlpc; i++) {
+                  qlpc[i] = var_get(LPCQUANT,this_shn);
+                  if (this_shn->vars.fatal_error) {
+                    retval = 0;
+                    goto got_enough_data;
+                  }
+                }
+                for(i = 0; i < nlpc; i++)
+                  cbuffer[i - nlpc] -= coffset;
+                for(i = 0; i < blocksize; i++)
+                {
+                  slong sum = lpcqoffset;
+
+                  for(j = 0; j < nlpc; j++)
+                    sum += qlpc[j] * cbuffer[i - j - 1];
+                  cbuffer[i] = var_get(resn,this_shn) + (sum >> LPCQUANT);
+                  if (this_shn->vars.fatal_error) {
+                    retval = 0;
+                    goto got_enough_data;
+                  }
+                }
+                if(coffset != 0)
+                  for(i = 0; i < blocksize; i++)
+                    cbuffer[i] += coffset;
+                break;
+            }
+
+            /* store mean value if appropriate : N.B. Duplicated code */
+            if(nmean > 0)
+            {
+              slong sum = (version < 2) ? 0 : blocksize / 2;
+
+              for(i = 0; i < blocksize; i++)
+                sum += cbuffer[i];
+
+              for(i = 1; i < nmean; i++)
+                offset[chan][i - 1] = offset[chan][i];
+              if(version < 2)
+                offset[chan][nmean - 1] = sum / blocksize;
+              else
+                offset[chan][nmean - 1] = (sum / blocksize) << bitshift;
+            }
+
+			if (0 == chan) {
+              this_shn->vars.initial_file_position = this_shn->vars.last_file_position_no_really;
+              goto got_enough_data;
+            }
+
+            /* do the wrap */
+            for(i = -nwrap; i < 0; i++)
+              cbuffer[i] = cbuffer[i + blocksize];
+
+            fix_bitshift(cbuffer, blocksize, bitshift, internal_ftype);
+
+            if(chan == nchan - 1)
+            {
+              fwrite_type(buffer, ftype, nchan, blocksize, this_shn);
+              this_shn->vars.bytes_in_buf = 0;
+            }
+
+            chan = (chan + 1) % nchan;
+            break;
+          }
+          break;
+
+          case FN_BLOCKSIZE:
+            UINT_GET((int) (log((double) blocksize) / M_LN2), this_shn);
+            break;
+
           case FN_VERBATIM:
             cklen = uvar_get(VERBATIM_CKSIZE_SIZE,this_shn);
 
             while (cklen--) {
               if (this_shn->vars.bytes_in_header >= OUT_BUFFER_SIZE) {
                   shn_debug("Unexpectedly large header - " PACKAGE " can only handle a maximum of %d bytes",OUT_BUFFER_SIZE);
-                  retval = 0;
                   goto got_enough_data;
               }
               this_shn->vars.bytes_in_buf = 0;
-	      this_shn->vars.header[this_shn->vars.bytes_in_header++] = (char)uvar_get(VERBATIM_BYTE_SIZE,this_shn);
+              this_shn->vars.header[this_shn->vars.bytes_in_header++] = (char)uvar_get(VERBATIM_BYTE_SIZE,this_shn);
             }
+            retval = 1;
+            break;
+
+          case FN_BITSHIFT:
+            bitshift = uvar_get(BITSHIFTSIZE,this_shn);
+            this_shn->vars.bitshift = bitshift;
             break;
 
           default:
@@ -422,6 +593,7 @@ void shn_unload(shn_file *this_shn)
 shn_file *load_shn(char *filename)
 {
 	shn_file *tmp_file;
+    shn_seek_entry *first_seek_table;
 
 	shn_debug("Loading file: '%s'",filename);
 
@@ -440,6 +612,13 @@ shn_file *load_shn(char *filename)
 	tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
 	tmp_file->vars.bytes_in_buf = 0;
 	tmp_file->vars.bytes_in_header = 0;
+	tmp_file->vars.reading_function_code = 0;
+	tmp_file->vars.initial_file_position = 0;
+	tmp_file->vars.last_file_position = 0;
+	tmp_file->vars.last_file_position_no_really = 0;
+	tmp_file->vars.bytes_read = 0;
+	tmp_file->vars.bitshift = 0;
+	tmp_file->vars.seek_offset = 0;
 
 	tmp_file->decode_state = NULL;
 
@@ -453,15 +632,17 @@ shn_file *load_shn(char *filename)
 	tmp_file->wave_header.rate = 0;
 	tmp_file->wave_header.header_size = 0;
 	tmp_file->wave_header.data_size = 0;
+	tmp_file->wave_header.file_has_id3v2_tag = 0;
+	tmp_file->wave_header.id3v2_tag_size = 0;
 
-	tmp_file->seek_header.version = 0;
+	tmp_file->seek_header.version = NO_SEEK_TABLE;
 	tmp_file->seek_header.shnFileSize = 0;
 
 	tmp_file->seek_trailer.seekTableSize = 0;
 
 	tmp_file->seek_table = NULL;
 
-	if (!(tmp_file->vars.fd = fopen(filename,"r")))
+	if (!(tmp_file->vars.fd = shn_open_and_discard_id3v2_tag(filename,&tmp_file->wave_header.file_has_id3v2_tag,&tmp_file->wave_header.id3v2_tag_size)))
 	{
 		shn_debug("Could not open file: '%s'",filename);
 		shn_unload(tmp_file);
@@ -475,7 +656,17 @@ shn_file *load_shn(char *filename)
 		return NULL;
 	}
 
-	fseek(tmp_file->vars.fd,0,SEEK_SET);
+	if (tmp_file->wave_header.file_has_id3v2_tag)
+	{
+		fseek(tmp_file->vars.fd,tmp_file->wave_header.id3v2_tag_size,SEEK_SET);
+		tmp_file->vars.bytes_read += tmp_file->wave_header.id3v2_tag_size;
+		tmp_file->vars.seek_offset = tmp_file->wave_header.id3v2_tag_size;
+	}
+    else
+	{
+		fseek(tmp_file->vars.fd,0,SEEK_SET);
+	}
+
 	if (0 == shn_verify_header(tmp_file))
 	{
 		shn_debug("Invalid WAVE header in file: '%s'",filename);
@@ -491,6 +682,30 @@ shn_file *load_shn(char *filename)
 
 	shn_load_seek_table(tmp_file,filename);
 
+	if (NO_SEEK_TABLE != tmp_file->vars.seek_table_entries)
+	{
+		first_seek_table = (shn_seek_entry *)tmp_file->seek_table;
+
+		/* check for broken seek tables - if found, disable seeking */
+		if (0 == tmp_file->seek_header.version)
+		{
+			/* test, if the bitshift value in the file is identical to the bitshift value of the first seektable entry */
+			if (tmp_file->vars.bitshift != shn_uchar_to_ushort_le(first_seek_table->data+22))
+			{
+				shn_debug("Broken seek table detected - seeking disabled for file '%s'.",tmp_file->wave_header.filename);
+				tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
+			}
+		}
+
+		tmp_file->vars.seek_offset += tmp_file->vars.initial_file_position - shn_uchar_to_ulong_le(first_seek_table->data+8);
+
+		if (0 != tmp_file->vars.seek_offset)
+		{
+			shn_debug("Adjusting seek table offsets by %ld bytes due to mismatch between seek table values and input file - seeking might not work correctly.",
+				tmp_file->vars.seek_offset);
+		}
+	}
+
 	shn_debug("Successfully loaded file: '%s'",filename);
 
 	return tmp_file;
@@ -502,7 +717,7 @@ static int shn_is_our_file(char *filename)
 	FILE *f;
 	char data[4];
 
-	if (!(f = fopen(filename,"r")))
+	if (!(f = shn_open_and_discard_id3v2_tag(filename,NULL,NULL)))
 		return FALSE;
 
 	if (fread((void *)data,1,4,f) != 4)
@@ -587,6 +802,7 @@ static void *play_loop_shn(void *arg)
   int   blk_size;
   int   cklen;
   uchar tmp;
+  ulong seekto_offset;
 
 restart:
 
@@ -623,7 +839,7 @@ restart:
       while(version > MAX_VERSION)
       {
   	int byte = getc(this_shn->vars.fd);
-	if(byte == EOF) {
+ 	if(byte == EOF) {
 	  shn_error_fatal(this_shn,"No magic number");
           goto exit_thread;
         }
@@ -890,19 +1106,24 @@ restart:
                 offset[0][1]  = shn_uchar_to_slong_le(seek_info->data+52);
                 offset[0][2]  = shn_uchar_to_slong_le(seek_info->data+56);
                 offset[0][3]  = shn_uchar_to_slong_le(seek_info->data+60);
-		if (nchan > 1)
-		{
-	                buffer[1][-1] = shn_uchar_to_slong_le(seek_info->data+36);
-        	        buffer[1][-2] = shn_uchar_to_slong_le(seek_info->data+40);
-                	buffer[1][-3] = shn_uchar_to_slong_le(seek_info->data+44);
-	                offset[1][0]  = shn_uchar_to_slong_le(seek_info->data+64);
-        	        offset[1][1]  = shn_uchar_to_slong_le(seek_info->data+68);
-                	offset[1][2]  = shn_uchar_to_slong_le(seek_info->data+72);
-	                offset[1][3]  = shn_uchar_to_slong_le(seek_info->data+76);
-		}
+
+                if (nchan > 1) {
+                  buffer[1][-1] = shn_uchar_to_slong_le(seek_info->data+36);
+                  buffer[1][-2] = shn_uchar_to_slong_le(seek_info->data+40);
+                  buffer[1][-3] = shn_uchar_to_slong_le(seek_info->data+44);
+                  offset[1][0]  = shn_uchar_to_slong_le(seek_info->data+64);
+                  offset[1][1]  = shn_uchar_to_slong_le(seek_info->data+68);
+                  offset[1][2]  = shn_uchar_to_slong_le(seek_info->data+72);
+                  offset[1][3]  = shn_uchar_to_slong_le(seek_info->data+76);
+                }
+
                 bitshift = shn_uchar_to_ushort_le(seek_info->data+22);
-                fseek(this_shn->vars.fd,(slong)shn_uchar_to_ulong_le(seek_info->data+8),SEEK_SET);
+
+                seekto_offset = shn_uchar_to_ulong_le(seek_info->data+8) + this_shn->vars.seek_offset;
+
+                fseek(this_shn->vars.fd,(slong)seekto_offset,SEEK_SET);
                 fread((uchar*) this_shn->decode_state->getbuf, 1, BUFSIZ, this_shn->vars.fd);
+
                 this_shn->decode_state->getbufp = this_shn->decode_state->getbuf + shn_uchar_to_ushort_le(seek_info->data+14);
                 this_shn->decode_state->nbitget = shn_uchar_to_ushort_le(seek_info->data+16);
                 this_shn->decode_state->nbyteget = shn_uchar_to_ushort_le(seek_info->data+12);
@@ -1111,13 +1332,16 @@ static void shn_get_file_info(char *filename, char **title, int *length)
 		temp = filename;
 	else
 		temp++;
+
 	name = g_malloc(strlen(temp) + 1);
 	strcpy(name, temp);
+
 	if (shn_filename_contains_a_dot(name))
 		*(strrchr(name,'.')) = '\0';
-	(*title) = name;
 
-	(*length) = 0;
+	*title = name;
+
+	*length = 0;
 
 	if (!(tmp_file = load_shn(filename)))
 	{
@@ -1125,7 +1349,7 @@ static void shn_get_file_info(char *filename, char **title, int *length)
 		return;
 	}
 
-	(*length) = 1000 * tmp_file->wave_header.length;
+	*length = 1000 * tmp_file->wave_header.length;
 
 	shn_unload(tmp_file);
 }
